//
// "$Id: ppdc-catalog.cxx 251 2005-06-17 02:53:12Z mike $"
//
//   Shared message catalog class for the CUPS PPD Compiler.
//
//   Copyright 2002-2005 by Easy Software Products.
//
//   These coded instructions, statements, and computer programs are the
//   property of Easy Software Products and are protected by Federal
//   copyright law.  Distribution and use rights are outlined in the file
//   "LICENSE.txt" which should have been included with this file.  If this
//   file is missing or damaged please contact Easy Software Products
//   at:
//
//       Attn: CUPS Licensing Information
//       Easy Software Products
//       44141 Airport View Drive, Suite 204
//       Hollywood, Maryland 20636 USA
//
//       Voice: (301) 373-9600
//       EMail: cups-info@cups.org
//         WWW: http://www.cups.org
//
// Contents:
//
//   ppdcCatalog::ppdcCatalog()   - Create a shared message catalog.
//   ppdcCatalog::~ppdcCatalog()  - Destroy a shared message catalog.
//   ppdcCatalog::add_message()   - Add a new message.
//   ppdcCatalog::find_message()  - Find a message in a catalog...
//   ppdcCatalog::load_messages() - Load messages from a .po file.
//   ppdcCatalog::save_messages() - Save the messages to a .po file.
//

//
// Include necessary headers...
//

#include "ppdc.h"


//
// 'ppdcCatalog::ppdcCatalog()' - Create a shared message catalog.
//

ppdcCatalog::ppdcCatalog(const char *f)	// I - Message catalog file
  : ppdcShared()
{
  messages = new ppdcArray();

  if (f)
    load_messages(f);
}


//
// 'ppdcCatalog::~ppdcCatalog()' - Destroy a shared message catalog.
//

ppdcCatalog::~ppdcCatalog()
{
  delete messages;
}


//
// 'ppdcCatalog::add_message()' - Add a new message.
//

void
ppdcCatalog::add_message(const char *id)// I - Message ID to add
{
  ppdcMessage	*m;			// Current message
  char		text[1024];		// Text to translate

  // Range check input...
  if (!id || !*id)
    return;

  // Verify that we don't already have the message ID...
  for (m = (ppdcMessage *)messages->first();
       m;
       m = (ppdcMessage *)messages->next())
    if (!strcmp(m->id->value, id))
      return;

  // Add the message...
  snprintf(text, sizeof(text), "TRANSLATE %s", id);
  messages->add(new ppdcMessage(id, text));
}


//
// 'ppdcCatalog::find_message()' - Find a message in a catalog...
//

const char *				// O - Message text
ppdcCatalog::find_message(const char *id)
					// I - Message ID
{
  ppdcMessage	*m;			// Current message


  for (m = (ppdcMessage *)messages->first();
       m;
       m = (ppdcMessage *)messages->next())
    if (!strcmp(m->id->value, id))
      return (m->string->value);

  return (id);
}


//
// 'ppdcCatalog::load_messages()' - Load messages from a .po file.
//

int					// O - 0 on success, -1 on failure
ppdcCatalog::load_messages(const char *f)
					// I - Message catalog file
{
  FILE		*fp;			// Message file
  ppdcMessage	*temp;			// Current message
  char		line[4096],		// Line buffer
		*ptr,			// Pointer into buffer
		id[4096];		// Translation ID
  int		linenum,		// Line number
		length;			// Length of combined strings


  if ((fp = fopen(f, "r")) == NULL)
    return (-1);

  id[0]   = '\0';
  linenum = 0;
  temp    = 0;

  while (fgets(line, sizeof(line), fp) != NULL)
  {
    linenum ++;

   /*
    * Skip comment lines...
    */

    if (line[0] == '#')
      continue;

   /*
    * Strip trailing newline and quote...
    */

    for (ptr = line + strlen(line) - 1;
         ptr >= line && isspace(*ptr);
	 *ptr-- = '\0');

    if (ptr < line)
      continue;

    if (*ptr != '\"')
    {
      fprintf(stderr, "load_messages: Expected quoted string on line %d!\n",
              linenum);
      fclose(fp);
      return (-1);
    }

    *ptr = '\0';

   /*
    * Find start of value...
    */
    
    if ((ptr = strchr(line, '\"')) == NULL)
    {
      fprintf(stderr, "load_messages: Expected quoted string on line %d!\n",
              linenum);
      fclose(fp);
      return (-1);
    }

    ptr ++;

    if (!strncmp(line, "msgid", 5))
    {
      temp = 0;
      strlcpy(id, ptr, sizeof(id));
    }
    else if (line[0] != '\"' && strncmp(line, "msgstr", 6))
    {
      fprintf(stderr, "load_messages: Unexpected text on line %d!\n", linenum);
      fclose(fp);
      return (-1);
    }
    else if (!id[0])
    {
      fprintf(stderr, "load_messages: Need a msgid line before any "
                      "translation strings on line %d!\n", linenum);
      fclose(fp);
      return (-1);
    }
    else if (temp)
    {
     /*
      * Append the string...
      */

      length              = strlen(temp->string->value) + strlen(ptr) + 1;
      temp->string->value = (char *)realloc(temp->string->value, length);

      strcat(temp->string->value, ptr);
    }
    else
    {
     /*
      * Set the string...
      */

      temp = new ppdcMessage(id, ptr);

      messages->add(temp);
    }
  }

  fclose(fp);

  return (0);
}


//
// 'ppdcCatalog::save_messages()' - Save the messages to a .po file.
//

int					// O - 0 on success, -1 on error
ppdcCatalog::save_messages(const char *f)
					// I - File to save to
{
  FILE		*fp;			// Message file
  ppdcMessage	*m;			// Current message
  const char	*ptr;			// Pointer into string


  if ((fp = fopen(f, "w")) == NULL)
    return (-1);

  for (m = (ppdcMessage *)messages->first();
       m;
       m = (ppdcMessage *)messages->next())
  {
    fputs("msgid \"", fp);
    for (ptr = m->id->value; *ptr; ptr ++)
      switch (*ptr)
      {
        case '\n' :
	    fputs("\\n", fp);
	    break;
        case '\\' :
	    fputs("\\\\", fp);
	    break;
        case '\"' :
	    fputs("\\\"", fp);
	    break;
        default :
	    putc(*ptr, fp);
	    break;
      }
    fputs("\"\n", fp);

    fputs("msgstr \"", fp);
    for (ptr = m->string->value; *ptr; ptr ++)
      switch (*ptr)
      {
        case '\n' :
	    fputs("\\n", fp);
	    break;
        case '\\' :
	    fputs("\\\\", fp);
	    break;
        case '\"' :
	    fputs("\\\"", fp);
	    break;
        default :
	    putc(*ptr, fp);
	    break;
      }
    fputs("\"\n", fp);

    putc('\n', fp);
  }

  fclose(fp);

  return (0);
}


//
// End of "$Id: ppdc-catalog.cxx 251 2005-06-17 02:53:12Z mike $".
//
